/*
 *      MessageID-Finder for Mozilla
 *
 *  Markus Hossner - markushossner@gmx.de
 *
 */

const pmpB = 0;
const pmpN = 1;
const pmpS = 2;
var prefRead = false;
var accountURIs = new Array();
var gMIDFinderStringBundle;
var msgWindowMIDFinder = Components.classes["@mozilla.org/messenger/msgwindow;1"].createInstance(Components.interfaces.nsIMsgWindow);


/*
 *   Finds message via messageid
 */
function MessageIdFinder(strhostname,strport,strmessageid)
{
  var openmsginnewwin   = pmpReadPref('extensions.messageidfinder.openmsginnewwin',pmpB);
  var defaultnewsserver = pmpReadPref('extensions.messageidfinder.defaultnewsserver',pmpS);

  var strMsgIdNotFound  = MIDFinderGetString("MsgIdNotFound");

  // *** If strmessageid is empty, get messageid ****
  if (strmessageid == "")
  {
    switch(document.firstChild.id)
    {
      // MessageID-Finder called within source view
      case "main-window":
        strmessageid = String(document.getElementById('content').contentWindow.getSelection());
        openmsginnewwin = true;
      break;

      // MessageID-Finder called within msgWindow
      case "messengerWindow":
        var source = document.popupNode.parentNode.localName;
        switch(source)
        {
          // MessageID-Finder called within message pane on selection
          case "DIV":
            source = document.getElementById('messagepane');
            strmessageid = String(source.contentWindow.getSelection());
          break;

          // MessageID-Finder called within message pane on link
          case "PRE":
            var linkText = gContextMenu.linkText();
            linkText = linkText.replace(/^<|>$/g, "");

            // Workaround for Bug 89939
            if (linkText.search(/news:[^\/]/) == 0) strmessageid  = linkText;
            else strmessageid = gContextMenu.linkURL();
          break;

          // MessageID-Finder called within message header
          case "label":
            strmessageid = document.popupNode.getAttribute("sMID");
          break;

          default:
            strmessageid = "";
          break;
        }
      break;

      // MessageID-Finder called within Menulist of PrefDialog
      case "messageidfinderprefdialog":
        strmessageid = "";
      break;

      // MessageID-Finder called within Menulist of InputDialog
      case "midinputdialog":
        strmessageid = "";
      break;

      default:
        strmessageid = "";
      break;
    }
  }

  // *** Extract messageid ***
  if (strmessageid != "")
  {
    strmessageid = strmessageid.replace(/^<|>$/g, "");

    var midParts = strmessageid.split(":");
    switch (midParts.length)
    {
      case 1:
        strmessageid = strmessageid;
      break;

      case 2:
        if (midParts[0] == "news" || midParts[0] == "nntp")
        {
          strmessageid = midParts[1].replace(/^\/\/.*?\//,"");
        }
        else if (midParts[0] == "mailto")
        {
          strmessageid = midParts[1];
        }
      break;

      case 3:
        strmessageid = midParts[2].replace(/^.+?\//,"");
      break;

      default:
        strmessageid = "";
      break;
    }
  }

  // *** Look for messageid ***
  var strsrc = "";
  if (strmessageid != "")
  {
    switch(strhostname)
    {

      // Look for MID locally
      case "local":
        var msgHdr = OpenMessageByMessageID(strmessageid);
        if (!msgHdr)
        {
          if (defaultnewsserver == "none" || defaultnewsserver == null)
          {
            alert(strMsgIdNotFound);
          }
          else if (defaultnewsserver.substring(0,6) == "google")
          {
            MessageIdFinder(defaultnewsserver,"",strmessageid);
          }
          else
          {
            strhostname = pmpReadPref("mail.server." + defaultnewsserver + ".hostname",pmpS);
            strport     = pmpReadPref("mail.server." + defaultnewsserver + ".port", pmpN);
            MessageIdFinder(strhostname,strport,strmessageid);
          }
        }
      break;

      // Look for MID with googl.com
      case "google.com":
         strsrc = 'http://groups.google.com/groups?selm=' + escape(strmessageid) + '&rnum=1';
      break;

      // Look for MID with googl.de
      case "google.de":
         strsrc = 'http://groups.google.de/groups?selm=' + escape(strmessageid) + '&rnum=1';
      break;

      // Look for MID with newsserver
      default:
         strsrc = 'news://' + strhostname + ':' + strport +'/'+ escape(strmessageid);
      break;

    }
  }

  // *** Open message ****
  if (strsrc.substring(0,4) == "news")
  {
    if (openmsginnewwin)
    {
      window.open(strsrc);
    }
    else
    {
      var collapsedHeaderViewNode, expandedHeaderViewNode;

      if (document.firstChild.id == "messengerWindow")
      {
        collapsedHeaderViewNode = window.document.getElementById("collapsedHeaderView");
        expandedHeaderViewNode  = window.document.getElementById("expandedHeaderView");
        window.content.location = strsrc;
      }
      else
      {
        collapsedHeaderViewNode = window.opener.document.getElementById("collapsedHeaderView");
        expandedHeaderViewNode  = window.opener.document.getElementById("expandedHeaderView");
        window.opener.content.location = strsrc;
      }

      if (collapsedHeaderViewNode) collapsedHeaderViewNode.collapsed = true;
      if (expandedHeaderViewNode) expandedHeaderViewNode.collapsed = true;
    }

  }
  else if (strsrc.substring(0,4) == "http")
  {
    window.open(strsrc);
  }

}


/*
 *   Opens and selects a message by messageid
 */
function OpenMessageByMessageID(messageId)
{
  var openmsginnewwin = pmpReadPref('extensions.messageidfinder.openmsginnewwin',pmpB);
  window.setCursor("wait");

  var msgHdr = SearchMessageLocally(messageId);
  if (msgHdr)
  {
    var folderUri  = msgHdr.folder.URI;

    if (openmsginnewwin)
    {
      var folder = GetMsgFolderFromUri(folderUri, true);
      var messageUri = folder.getUriForMsg(msgHdr);

      window.openDialog("chrome://messenger/content/messageWindow.xul", "_blank", "all,chrome,dialog=no,status,toolbar", messageUri, folderUri, null);
    }
    else
    {
      try
      {
        var indicesArray = {};
        var length = {};
        var tree;

        var msgWindow = Components.classes["@mozilla.org/messenger/msgwindow;1"].createInstance();
        msgWindow = msgWindow.QueryInterface(Components.interfaces.nsIMsgWindow);
        if (document.firstChild.id == "messengerWindow") msgWindow.SetDOMWindow(window);
        else msgWindow.SetDOMWindow(window.opener);
        msgWindow.SelectFolder(folderUri);

        gDBView.loadMessageByMsgKey(msgHdr.messageKey);
        gDBView.selectMsgByKey(msgHdr.messageKey);
        gDBView.getIndicesForSelection(indicesArray,length);
        tree = GetThreadTree();
        tree.treeBoxObject.ensureRowIsVisible(indicesArray.value[0]);
      }
      catch (ex) {}
    }
  }

 window.setCursor("auto");
 return msgHdr;
}


/*
 *   Searchs for messageid in local folders
 */
function SearchMessageLocally(messageId)
{
  var msgHdr = null;
  var folder = null;

  var x = 0;
  do
  {
    folder = GetMsgFolderFromUri(accountURIs[x], true).rootFolder;
    msgHdr = SearchInSubFolder(folder, messageId);
    x++;
  }
  while(!msgHdr && x < accountURIs.length);

  return msgHdr;
}


/*
 *   Searchs in subfolder
 */
function SearchInSubFolder(folder, messageId)
{
  if (folder.hasSubFolders)
  {
    var subfolders = folder.GetSubFolders();
    var subfolder = null;
    var msgHdr = null;
    var msgDB = null;
    var currentFolderURI = "";

    var done = false;
    while(!done)
    {
      currentFolderURI = subfolders.currentItem().QueryInterface(Components.interfaces.nsIRDFResource).Value;
      subfolder = GetMsgFolderFromUri(currentFolderURI, true);

      if (currentFolderURI.substring(1,7) != "news://")  msgHdr = SearchInSubFolder(subfolder, messageId);

      msgDB = subfolder.getMsgDatabase(msgWindowMIDFinder);
      if (!msgHdr) msgHdr = msgDB.getMsgHdrForMessageID(messageId);
      if (msgHdr) return msgHdr;

      try
      {
        subfolders.next();
      }
      catch (e)
      {
        done = true;
      }

    }
  }
 return null;
}


/*
 *   Searchs all availible newsserver
 */
function AddNewsServers()
{
  if (prefRead == false)
  {
    var accounts = pmpReadPref("mail.accountmanager.accounts",pmpS);

    if (accounts)
    {
      var accountsArray = accounts.split(",");
      for (var x=0; x<accountsArray.length; x++)
      {
        AddNewsServerMenuItem(accountsArray[x]);
      }
    }

    prefRead = true;
  }
}


/*
 *   Adds newsserver to the context menu
 */
function AddNewsServerMenuItem(account)
{
  var messageidfindermenu = document.getElementById("messageidfinder-popup");

  var server    = pmpReadPref("mail.account." + account + ".server",pmpS);
  var servertyp = pmpReadPref("mail.server." + server + ".type",pmpS);
  var hostname  = pmpReadPref("mail.server." + server + ".hostname",pmpS);
  var username  = pmpReadPref("mail.server." + server + ".userName",pmpS);

  var accountURI = "";
  if (servertyp == "nntp")
  {
    var servername = pmpReadPref("mail.server." + server + ".name",pmpS);
    var serverport = pmpReadPref("mail.server." + server + ".port", pmpN);
    var newsservermenuitem;

    if (servername == null) servername = hostname;
    if (serverport == null) serverport = 119;

    newsservermenuitem = document.createElement("menuitem");
    newsservermenuitem.setAttribute("label", servername);
    newsservermenuitem.setAttribute("value",server);
    newsservermenuitem.setAttribute("oncommand","MessageIdFinder('" + hostname + "', '" + String(serverport) + "','');");
    messageidfindermenu.appendChild(newsservermenuitem);

    accountURI = "news://" + escape(username) + "@" + escape(hostname);
  }
  else if (servertyp == "imap")
  {
    accountURI = "imap://" + escape(username) + "@" + escape(hostname);
  }
  else
  {
    accountURI = "mailbox://" + escape(username) + "@" + escape(hostname);
  }

  accountURIs.push(accountURI);
}


/*
 *   Reads the value of pmpName out of the mozilla preferences
 */
function pmpReadPref(pmpName, pmpType)
{
   var nsIPrefBranch = Components.interfaces.nsIPrefBranch;
   var pref = Components.classes[ "@mozilla.org/preferences;1" ].getService(nsIPrefBranch);
   var pmpValue = null;

   if (pref.prefHasUserValue(pmpName))
   {
     switch ( pmpType )
     {
       case pmpB:
         pmpValue = pref.getBoolPref(pmpName);
         break;
       case pmpN:
         pmpValue = pref.getIntPref(pmpName);
         break;
       case pmpS:
         pmpValue = pref.getCharPref(pmpName);
         break;
     }
   }
   return pmpValue;
}


/*
 *   Get msgfolder from Uri
 */
function GetMsgFolderFromUri(uri, checkFolderAttributes)
{
  var msgfolder = null;
  try
  {
    var RDF = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService();
    RDF = RDF.QueryInterface(Components.interfaces.nsIRDFService);
    var resource = RDF.GetResource(uri);

    msgfolder = resource.QueryInterface(Components.interfaces.nsIMsgFolder);
    if (checkFolderAttributes)
    {
      if (!(msgfolder && (msgfolder.parent || msgfolder.isServer)))
      {
        msgfolder = null;
      }
    }
  }
  catch (ex) {}

  return msgfolder;
}


/*
 *   Get string from stringbundle
 */
function MIDFinderGetString(name)
{
  if (!gMIDFinderStringBundle)
  {
    try
    {
      var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService();
      strBundleService =  strBundleService.QueryInterface(Components.interfaces.nsIStringBundleService);
      gMIDFinderStringBundle = strBundleService.createBundle("chrome://messageidfinder/locale/messageidfinder.properties");
    }
    catch (ex) {}
  }

  if (gMIDFinderStringBundle)
  {
    try
    {
      return gMIDFinderStringBundle.GetStringFromName(name);
    }
    catch (e) {}
  }

  return null;
}